package com.ruoyi.web.controller;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.web.entity.DeepSeekLog;
import com.ruoyi.web.service.DeepSeekLogService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * deepseek日志表(DeepSeekLog)表控制层
 *
 * @author wu
 * @since 2025-03-09 20:28:04
 */
@RestController
@RequestMapping("/deepSeekLog")
@Api(tags = "deepseek日志表")
@Transactional(rollbackFor = Exception.class)
public class DeepSeekLogController extends BaseController {

    @Resource
    private RedisCache redisCache;
    /**
     * 服务对象
     */
    @Resource
    private DeepSeekLogService deepSeekLogService;

    @Resource
    private ISysUserService userService;

    /**
     * 分页条件查询所有数据
     *
     * @param page        分页条件
     * @param deepSeekLog 查询条件
     * @return 所有数据
     */
    @GetMapping
    @ApiOperation(value = "分页条件查询deepseek日志表", response = DeepSeekLog.class)
    public AjaxResult page(Page<DeepSeekLog> page, DeepSeekLog deepSeekLog) {
        return success(deepSeekLogService.page(page, new QueryWrapper<>(deepSeekLog)));
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @GetMapping("{id}")
    @ApiOperation(value = "通过主键查询单条deepseek日志表", response = DeepSeekLog.class)
    public AjaxResult getById(@PathVariable Serializable id) {
        return success(deepSeekLogService.getById(id));
    }

    /**
     * 根据用户id查询deepSeek使用次数
     *
     * @return 单条数据
     */
    @GetMapping("findByUserId")
    @ApiOperation(value = "根据用户id查询deepSeek使用次数")
    public AjaxResult findByUserId() {
        Long userId = SecurityUtils.getUserId();
        if (userId == null) {
            throw new ServiceException("用户未登录");
        }
        SysUser sysUser = userService.selectUserById(userId);
        if (BeanUtil.isEmpty(sysUser)) {
            throw new ServiceException("用户未登录");
        }
        Map<String, Object> map = redisCache.getCacheMap(sysUser.getUserName() + "DeepSeek");
        int dailyVisits = 0;
        if (CollectionUtil.isNotEmpty(map)) {
            Date latestAccessTime = (Date) map.get("latestAccessTime");
            // 判断时间是否是当天时间
            if (DateUtil.isSameDay(latestAccessTime, new Date())) {
                dailyVisits = (int) map.get("dailyVisits");
            }
        }
        return success(dailyVisits);
    }

    /**
     * 新增数据
     *
     * @return 新增结果
     */
    @PostMapping
    @ApiOperation(value = "新增deepseek日志表", response = DeepSeekLog.class)
    public AjaxResult insert() {
        Long userId = SecurityUtils.getUserId();
        if (userId == null) {
            throw new ServiceException("用户未登录");
        }
        SysUser sysUser = userService.selectUserById(userId);
        if (BeanUtil.isEmpty(sysUser)) {
            throw new ServiceException("用户未登录");
        }
        String username = sysUser.getUserName();
        int dailyVisits = 1;
        int totalVisits = 1;
        Map<String, Object> map = redisCache.getCacheMap(username + "DeepSeek");
        if (CollectionUtil.isNotEmpty(map)) {
            Date latestAccessTime = (Date) map.get("latestAccessTime");
            // 判断时间是否是当天时间
            if (DateUtil.isSameDay(latestAccessTime, new Date())) {
                dailyVisits = (int) map.get("dailyVisits") + 1;
                if (dailyVisits > 50) {
                    throw new ServiceException("用户试用次数已达上限");
                }
                map.put("dailyVisits", dailyVisits);
                map.put("latestAccessTime", new Date());
                redisCache.setCacheMap(username + "DeepSeek", map);
            } else {
                HashMap<String, Object> redisMap = new HashMap<>();
                redisMap.put("dailyVisits", dailyVisits);
                redisMap.put("latestAccessTime", new Date());
                redisCache.setCacheMap(username + "DeepSeek", redisMap);
            }
        } else {
            HashMap<String, Object> redisMap = new HashMap<>();
            redisMap.put("dailyVisits", dailyVisits);
            redisMap.put("latestAccessTime", new Date());
            redisCache.setCacheMap(username + "DeepSeek", redisMap);
        }
        // 获取历史记录
        QueryWrapper<DeepSeekLog> wrapper = new QueryWrapper<>();
        wrapper.eq("username", username);
        wrapper.eq("user_id", userId);
        DeepSeekLog one = deepSeekLogService.getOne(wrapper);
        if (BeanUtil.isNotEmpty(one)) {
            totalVisits = one.getTotalVisits() + 1;
            one.setLatestAccessTime(new Date());
            one.setTotalVisits(totalVisits);
            one.setDailyVisits(dailyVisits);
            return success(deepSeekLogService.updateById(one));
        } else {
            DeepSeekLog deepSeekLog = new DeepSeekLog();
            deepSeekLog.setUsername(username);
            deepSeekLog.setUserId(userId);
            deepSeekLog.setLatestAccessTime(new Date());
            deepSeekLog.setTotalVisits(totalVisits);
            deepSeekLog.setDailyVisits(dailyVisits);
            return success(deepSeekLogService.save(deepSeekLog));
        }
    }

    /**
     * 修改数据
     *
     * @param deepSeekLog 实体对象
     * @return 修改结果
     */
    @PutMapping
    @ApiOperation(value = "修改deepseek日志表")
    public AjaxResult update(@RequestBody DeepSeekLog deepSeekLog) {
        return success(deepSeekLogService.updateById(deepSeekLog));
    }

    /**
     * 删除数据
     *
     * @param idList 主键集合
     * @return 删除结果
     */
    @DeleteMapping
    @ApiOperation(value = "删除deepseek日志表")
    public AjaxResult delete(@RequestParam("idList") List<Long> idList) {
        return success(deepSeekLogService.removeByIds(idList));
    }
}

